/* morseplay - play morse code signals via sndio
 * pesco 2021
 *
 * reads dots (.) and dashes (-) on stdin and plays dits and dahs accordingly.
 * precise timing via the output sound stream, including attack/decay envelope.
 *
 * compatible with the output of 'morse -s':
 * whitespace produces an intra-word pause (three dit lengths).
 * empty lines produce an inter-word pause (seven dit lengths).
 *
 * output speed is selectable in words per minute (PARIS standard).
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>	/* getopt() */
#include <string.h>	/* strcmp() */
#include <err.h>	/* warnx() */
#include "cw.h"

/* settings */
int speed = 24;			/* base speed [wpm] */
int pspeed = 24;		/* speed used for pauses [wpm] */
int freq = 600;			/* tone pitch [Hz] */
int echo = 0;

int
main(int argc, char **argv)
{
	int ch;
	int len, plen;		/* dit length, separately for pauses [ms] */

	/* very crude input parser state machine :) */
	int nl = 1;	/* new line of input? */
	int ws = 0;	/* whitespace seen? */
	int dd = 0;	/* dot or dash seen? */

	setvbuf(stdout, NULL, _IONBF, 0);	/* disable output buffering */

	/* command line arguments */
	while ((ch = getopt(argc, argv, "eh")) != -1) {
		switch (ch) {
		case 'e':
			echo = 1;
			break;
		case 'h':
		default:
			printf("usage: morseplay [-eh]\n");
			return 0;
		}
	}

	/* environment variables */
	getenvnum("MORSE_SPEED", "wpm", &speed);
	pspeed = speed;	/* default */
	getenvnum("MORSE_PAUSE_SPEED", "wpm", &pspeed);
	getenvnum("MORSE_PITCH", "Hz", &freq);

	/* derive base (dit) lengths from speed settings */
	len = 1200 / speed;
	plen = 1200 / pspeed;

	cwinit();

	/* main loop */
	cwstart();
	while ((ch = getchar()) != EOF) {
		switch(ch) {
		case '.':
			if (dd)
				silence(len);
			beep(freq, len);
			dd = 1;
			nl = ws = 0;
			break;
		case '-':
			if (dd)
				silence(len);
			beep(freq, 3 * len);
			dd = 1;
			nl = ws = 0;
			break;
		case ' ':
		case '\t':
			if (!ws)
				silence(3 * plen);
			ws = 1;
			dd = 0;
			break;
		case '\n':
			if (nl)
				silence(7 * plen);
			else if (!ws)
				silence(3 * plen);
			nl = ws = 1;
			dd = 0;
			break;
		default:
			continue;	/* do not echo */
		}

		if (echo)
			putchar(ch);
	}
	cwstop();

	return 0;
}
